@@ -24,6 +24,8 @@
 #include <string.h>
 #include <unistd.h>
 #include <arpa/inet.h>
+#include <openssl/crypto.h>
+#include <openssl/err.h>
 
 #include "util.h"
 #include "duo.h"
@@ -165,15 +167,61 @@ do_auth(struct login_ctx *ctx, const char *cmd)
         }
         return (EXIT_FAILURE);
     }
+    
+
+#ifdef OPENSSL_FIPS
+    /*
+     * When fips_mode is configured, invoke OpenSSL's FIPS_mode_set() API. Note
+     * that in some environments, FIPS may be enabled system-wide, causing FIPS
+     * operation to be enabled automatically when OpenSSL is initialized.  The
+     * fips_mode option is an experimental feature allowing explicit entry to FIPS
+     * operation in cases where it isn't enabled globally at the OS level (for
+     * example, when integrating directly with the OpenSSL FIPS Object Module).
+     */
+    if(!FIPS_mode_set(cfg.fips_mode)) {
+        /* The smallest size buff can be according to the openssl docs */ 
+        char buff[256];
+        int error = ERR_get_error();
+        ERR_error_string_n(error, buff, sizeof(buff));
+        duo_syslog(LOG_ERR, "Unable to start fips_mode: %s", buff);
+	 
+       return (EXIT_FAILURE);
+    }
+#else
+    if(cfg.fips_mode) {
+        duo_syslog(LOG_ERR, "FIPS mode flag specified, but OpenSSL not built with FIPS support. Failing the auth.");
+        return (EXIT_FAILURE);
+    }
+#endif
+
     prompts = cfg.prompts;
     /* Check group membership. */
     matched = duo_check_groups(pw, cfg.groups, cfg.groups_cnt);
     if (matched == -1) {
+        close_config(&cfg);
         return (EXIT_FAILURE);
     } else if (matched == 0) {
+        duo_syslog(LOG_INFO, "User %s bypassed Duo 2FA due to user's UNIX group", duouser);
+        close_config(&cfg);
         return (EXIT_SUCCESS);
     }
 
+    /* Use GECOS field if called for */
+    if ((cfg.send_gecos || cfg.gecos_username_pos >= 0) && !ctx->duouser) {
+        if (strlen(pw->pw_gecos) > 0) {
+            if (cfg.gecos_username_pos >= 0) {
+                duouser = duo_split_at(pw->pw_gecos, cfg.gecos_delim, cfg.gecos_username_pos);
+                if (duouser == NULL || (strcmp(duouser, "") == 0)) {
+                    duo_log(LOG_DEBUG, "Could not parse GECOS field", pw->pw_name, NULL, NULL);
+                    duouser = pw->pw_name;
+                }
+            } else {
+                duouser = pw->pw_gecos;
+            }
+        } else {
+            duo_log(LOG_WARNING, "Empty GECOS field", pw->pw_name, NULL, NULL);
+        }
+    }
 
     /* Check for remote login host */
     if ((host = ip = getenv("SSH_CONNECTION")) != NULL ||
@@ -196,6 +244,7 @@ do_auth(struct login_ctx *ctx, const char *cmd)
                     cfg.https_timeout, cfg.http_proxy)) == NULL) {
         duo_log(LOG_ERR, "Couldn't open Duo API handle",
             pw->pw_name, host, NULL);
+        close_config(&cfg);
         return (EXIT_FAILURE);
     }
 
@@ -220,7 +269,7 @@ do_auth(struct login_ctx *ctx, const char *cmd)
 
     for (i = 0; i < prompts; i++) {
         code = duo_login(duo, duouser, host, flags,
-                    cfg.pushinfo ? cmd : NULL);
+                    cfg.pushinfo ? cmd : NULL, cfg.failmode);
         if (code == DUO_FAIL) {
             duo_log(LOG_WARNING, "Failed Duo login",
                 duouser, host, duo_geterr(duo));
@@ -251,12 +300,13 @@ do_auth(struct login_ctx *ctx, const char *cmd)
         } else if (code == DUO_ABORT) {
             duo_log(LOG_WARNING, "Aborted Duo login",
                 duouser, host, duo_geterr(duo));
-        } else if (cfg.failmode == DUO_FAIL_SAFE &&
-                    (code == DUO_CONN_ERROR ||
-                     code == DUO_CLIENT_ERROR || code == DUO_SERVER_ERROR)) {
+        } else if (code == DUO_FAIL_SAFE_ALLOW) {
             duo_log(LOG_WARNING, "Failsafe Duo login",
                 duouser, host, duo_geterr(duo));
                         ret = EXIT_SUCCESS;
+        } else if (code == DUO_FAIL_SECURE_DENY) {
+            duo_log(LOG_WARNING, "Failsecure Duo login",
+                duouser, host, duo_geterr(duo));
         } else {
             duo_log(LOG_ERR, "Error in Duo login",
                 duouser, host, duo_geterr(duo));
@@ -264,6 +314,7 @@ do_auth(struct login_ctx *ctx, const char *cmd)
         break;
     }
     duo_close(duo);
+    close_config(&cfg);
 
     return (ret);
 }
