@@ -3,6 +3,7 @@
 import os
 import subprocess
 import sys
+import platform
 
 import paths
 
@@ -14,6 +15,9 @@ def main():
         env['DYLD_INSERT_LIBRARIES'] = paths.build + \
                                        '/.libs/libgroups_preload.dylib'
         env['DYLD_FORCE_FLAT_NAMESPACE'] = '1'
+    elif sys.platform == 'sunos5':
+        architecture = {'32bit': '32', '64bit': '64'}[platform.architecture()[0]]
+        env['LD_PRELOAD_' + architecture] = paths.build + '/.libs/libgroups_preload.so'
     else:
         env['LD_PRELOAD'] = paths.build + '/.libs/libgroups_preload.so'
 
